__all__ = [ "action",
            "add_action",
            "clean_action",
            "clone_action",
            "config_action",
            "download_action",
            "filters_action",
            "import_action",
            "init_action",
            "list_action",
            "move_action",
            "pull_action",
            "push_action",
            "request_action",
            "rm_action",
            "status_action",
            "reference_action"
          ]
