CONF_DIR = '.ltk'
CONF_FN = 'config'
TRANS_DIR = 'Lingotek Translations'
DB_FN = 'docs.json'
FOLDER_DB_FN = 'folders.json'
LOG_FN = 'log'
APP_ID = 'ab33b8b9-4c01-43bd-a209-b59f933e4fc4'
SYSTEM_FILE = '.lingotek'
ERROR_FN = 'error_log'
METADATA_FIELDS = ['author_email', 'author_name', 'business_division', 'business_unit', 'campaign_id', 'campaign_rating', 'channel', 'contact_email', 'contact_name', 'content_description', 'content_type', 'domain', 'external_application_id', 'external_document_id', 'external_style_id', 'job_id', 'purchase_order', 'reference_url', 'region', 'require_review', 'note']